package com.qmth.themis.mq.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.Gson;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.entity.TOeExamBreakHistory;
import com.qmth.themis.business.enums.BreakReasonEnum;
import com.qmth.themis.business.enums.FinishTypeEnum;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.enums.SystemOperationEnum;
import com.qmth.themis.business.service.*;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.templete.impl.TaskExamPaperImportTemplete;
import com.qmth.themis.business.templete.impl.TaskExamStudentImportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeExportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeImportTemplete;
import com.qmth.themis.business.threadPool.MyThreadPool;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.service.MqLogicService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;

/**
 * @Description: mq执行逻辑 impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/31
 */
@Service
public class MqLogicServiceImpl implements MqLogicService {

    @Resource
    RedisUtil redisUtil;

    @Resource
    TMRocketMessageService tmRocketMessageService;

    @Resource
    TEUserLogService teUserLogService;

    @Resource
    TEExamStudentLogService teExamStudentLogService;

    @Resource
    TBSessionService tbSessionService;

    @Resource
    MyThreadPool myThreadPool;

    @Resource
    TOeExamRecordService tOeExamRecordService;

    @Resource
    TOeExamBreakHistoryService tOeExamBreakHistoryService;

    @Resource
    TOeExamRecordService examRecordService;

    @Resource
    TOeFaceVerifyHistoryService faceVerifyHistoryService;

    @Resource
    TOeLivenessVerifyHistoryService livenessVerifyHistoryService;
    @Resource
    TEExamService teExamService;

    /*** mq最大重试次数逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqMaxReconsumeTime(MqDto mqDto, String key) {
        //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
        Gson gson = new Gson();
        mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 用户轨迹逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqUserLogLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        String tag = mqDto.getTag();
        if (tag.contains("user")) {
            teUserLogService.saveUserLogInfo(mqDto.getTimestamp(), mqDto.getObjId(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
        } else if (tag.contains("student")) {
            teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
        }
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * session逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqSessionLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        tbSessionService.saveSessionInfo(JacksonUtil.readJson(JacksonUtil.parseJson(mqDto.getBody()), TBSession.class), mqDto.getTimestamp());
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * task逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqTaskLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> map = (Map<String, Object>) mqDto.getBody();
        String tag = mqDto.getTag();
        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
            if (tag.contains("Import")) {
                TaskImportTemplete taskImportTemplete = null;
                if (tag.contains("examStudentImport")) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskExamStudentImportTemplete.class);
                } else if (tag.contains("roomCodeImport")) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskRoomCodeImportTemplete.class);
                } else if (tag.contains("examPaperImport")) {
                    taskImportTemplete = SpringContextHolder.getBean(TaskExamPaperImportTemplete.class);
                }
                try {
                    taskImportTemplete.importTask(map);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                TaskExportTemplete taskExportTemplete = null;
                if (tag.contains("roomCodeExport")) {
                    taskExportTemplete = SpringContextHolder.getBean(TaskRoomCodeExportTemplete.class);
                }
                try {
                    taskExportTemplete.exportTask(map);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        });
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        mqDto.setBody(JacksonUtil.parseJson(mqDto.getBody()));
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * websocket超时退出逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqWebsocketUnNormalLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);//表示成功处理
        Map<String, Object> tranMap = mqDto.getProperties();
        Long recordId = Long.parseLong(String.valueOf(tranMap.get("recordId")));
        Date clientLastSyncTime = ExamRecordCacheUtil.getClientLastSyncTime(recordId);
        if (Objects.nonNull(clientLastSyncTime) && (System.currentTimeMillis() - clientLastSyncTime.getTime()) / 1000 / 60 >= 2) {//大于等于当前时间，说明未重连或重登录
            String deviceId = String.valueOf(tranMap.get("deviceId"));
            String ip = String.valueOf(tranMap.get("ip"));
            Long updateTime = Long.parseLong(String.valueOf(tranMap.get("updateTime")));
            Date date = new Date();
            date.setTime(updateTime);
//            TOeExamRecord tOeExamRecord = tOeExamRecordService.getById(recordId);
            Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
//            Integer breakCount = tOeExamRecord.getLeftBreakResumeCount();
            Integer leftBreakResumeCount = ExamRecordCacheUtil.getLeftBreakResumeCount(recordId);
            Long examId = Long.parseLong(String.valueOf(objectMap.get("examId")));
            Long examStudentId = Long.parseLong(String.valueOf(objectMap.get("examStudentId")));
            Integer durationSeconds = Integer.parseInt(String.valueOf(objectMap.get("durationSeconds")));
//            获取考试缓存
//            ExamCacheBean ec = teExamService.getExamCacheBean(examId);
            if (Objects.isNull(leftBreakResumeCount) || leftBreakResumeCount <= 0) {
//                tOeExamRecordService.updateById(tOeExamRecord);
                teExamService.finish(examStudentId, recordId, FinishTypeEnum.AUTO.name(), durationSeconds);
            }
//            else {//否则断点次数减1
//                leftBreakResumeCount--;
//                //如果断点次数超过了考试整体断点次数，也强制交卷
//                if (leftBreakResumeCount > ec.getBreakResumeCount()) {
//                    teExamService.finish(examStudentId, recordId, FinishTypeEnum.AUTO.name(), durationSeconds);
//                } else {
//                    //先查询之前的断点记录
//                    QueryWrapper<TOeExamBreakHistory> tOeExamBreakHistoryQueryWrapper = new QueryWrapper<>();
//                    tOeExamBreakHistoryQueryWrapper.lambda().eq(TOeExamBreakHistory::getExamRecordId, recordId);
//                    List<TOeExamBreakHistory> tOeExamBreakHistoryList = tOeExamBreakHistoryService.list(tOeExamBreakHistoryQueryWrapper);
//                    //删除历史断点缓存
//                    if (Objects.nonNull(tOeExamBreakHistoryList) && tOeExamBreakHistoryList.size() > 0) {
//                        tOeExamBreakHistoryList.forEach(s -> {
//                            redisUtil.delete(SystemConstant.EXAM_BREAK + String.valueOf(s.getId()));
//                        });
//                    }
//                    //增加断点记录
//                    TOeExamBreakHistory tOeExamBreakHistory = new TOeExamBreakHistory(recordId, new Date(), BreakReasonEnum.NET_TIME_OUT, BreakReasonEnum.NET_TIME_OUT.name());
//                    tOeExamBreakHistoryService.save(tOeExamBreakHistory);
//                    redisUtil.set(SystemConstant.EXAM_BREAK + String.valueOf(tOeExamBreakHistory.getId()), tOeExamBreakHistory);
//                    //更新考试记录状态
////                    tOeExamRecord.setClientCurrentIp(ip);
////                    tOeExamRecord.setClientWebsocketId(deviceId);
////                    tOeExamRecord.setClientWebsocketStatus(WebsocketStatusEnum.UN_ONLINE.ordinal());
////                    tOeExamRecord.setClientLastSyncTime(date);
////                    tOeExamRecord.setLastBreakId(tOeExamBreakHistory.getId());
////                    tOeExamRecord.setLastBreakTime(tOeExamBreakHistory.getBreakTime());
////                    tOeExamRecord.setLeftBreakResumeCount(breakCount);
////                    tOeExamRecord.setStatus(ExamRecordStatusEnum.bREAK_OFF);
//                }
////                tOeExamRecordService.updateById(tOeExamRecord);
//            }
        }
        teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        Map map = new HashMap();
        map.put(SystemConstant.MQDTO_OBJ, JacksonUtil.parseJson(mqDto));
        tmRocketMessage.setProp(JacksonUtil.parseJson(map));
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 计算客观分逻辑
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqCalculateObjectiveScoreLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        examRecordService.calculateObjectiveScore(param);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 人脸验证保存
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqFaceVerifySaveLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long id = (Long) param.get("id");
        Long recordId = (Long) param.get("recordId");
        String type = (String) param.get("type");
        String photoUrl = (String) param.get("photoUrl");
        Integer faceCount = (Integer) param.get("faceCount");
        Double similarity = (Double) param.get("similarity");
        Double realness = (Double) param.get("realness");
        Long time = (Long) param.get("time");
        String exception = (String) param.get("exception");
        faceVerifyHistoryService.save(id, recordId, type, photoUrl, faceCount, similarity, realness, time, exception);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 活体验证
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqLivenessVerifySaveLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long id = (Long) param.get("id");
        Long recordId = (Long) param.get("recordId");
        String type = (String) param.get("type");
        String actions = (String) param.get("actions");
        Integer retry = (Integer) param.get("retry");
        Long startTime = (Long) param.get("startTime");
        Long finishTime = (Long) param.get("finishTime");
        Boolean pass = (Boolean) param.get("pass");
        livenessVerifyHistoryService.save(id, recordId, type, actions, retry, startTime, finishTime, pass);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 考试记录数据持久化
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqExamRecordPersistedLogic(MqDto mqDto, String key) {
        Gson gson = new Gson();
        Map<String, Object> param = (Map<String, Object>) mqDto.getBody();
        Long recordId = (Long) param.get("recordId");
        examRecordService.persisted(recordId);
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }

    /**
     * 增加考试断点记录
     *
     * @param mqDto
     * @param key
     */
    @Override
    @Transactional
    public void execMqExamBreakHistoryLogic(MqDto mqDto, String key) {
        //更新考试记录
        Map<String, Object> objectMap = JacksonUtil.readJson(String.valueOf(mqDto.getBody()), Map.class);
        Long recordId = Long.parseLong(mqDto.getObjId());

        //先查询之前的断点记录
        QueryWrapper<TOeExamBreakHistory> tOeExamBreakHistoryQueryWrapper = new QueryWrapper<>();
        tOeExamBreakHistoryQueryWrapper.lambda().eq(TOeExamBreakHistory::getExamRecordId, recordId);
        List<TOeExamBreakHistory> tOeExamBreakHistoryList = tOeExamBreakHistoryService.list(tOeExamBreakHistoryQueryWrapper);
        //删除历史断点缓存
        if (Objects.nonNull(tOeExamBreakHistoryList) && tOeExamBreakHistoryList.size() > 0) {
            tOeExamBreakHistoryList.forEach(s -> {
                redisUtil.delete(SystemConstant.EXAM_BREAK + String.valueOf(s.getId()));
            });
        }
        //增加断点记录
        TOeExamBreakHistory tOeExamBreakHistory = new TOeExamBreakHistory(recordId, new Date(), BreakReasonEnum.NET_TIME_OUT, BreakReasonEnum.NET_TIME_OUT.name());
        tOeExamBreakHistoryService.save(tOeExamBreakHistory);
        redisUtil.set(SystemConstant.EXAM_BREAK + String.valueOf(tOeExamBreakHistory.getId()), tOeExamBreakHistory);

        objectMap.put("lastBreakId", tOeExamBreakHistory.getId());
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(recordId), objectMap);

        Gson gson = new Gson();
        teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
        tmRocketMessage.setBody(JacksonUtil.parseJson(tmRocketMessage.getBody()));
        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
        redisUtil.delete(key, mqDto.getId());
    }
}
